/*------------------------------------------------------------------------------
    GALE. Fine-grained parallel Genetic Algorithms for Data Mining.
    Copyright (C) 2002  Xavier Llor <xllora@illigal.ge.uiuc.edu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
------------------------------------------------------------------------------*/


package edu.uiuc.ge.illigal.gale.data;

import java.util.Vector;
import java.util.Hashtable;

/**
 * Attribute class contains the information for an attribute of the data set.
 *
 * @author  Xavier Llor&agrave; &lt;<a href="mailto:xllora@illigal.ge.uiuc.edu">xllora@illigal.ge.uiuc.edu</a>&gt;
 * @version 0.9alpha
 * @since 0.9alpha
 */

public class Attribute
{
   //---------------------------------------------------------------------

	/** Constant definition for a nominal attribute */
	public final static int NOMINAL = 0;

	/** Constant definition for a real-valued attribute */
	public final static int REAL = 1;

	/** Constant definition for a integer-valued attribute */
	public final static int INTEGER = 2;

   //---------------------------------------------------------------------

	/** The attribute name */
	protected String sName;

	/** Type of the attribute */
	protected int iType;

	/** Maximum value of the attribute */
	protected float fMaxValue;
	
	/** Minimum value of the attribute */
	protected float fMinValue;

	/** Upper bound of the attribute */
	protected float fUpperBound;
	
	/** Lower bound of the attribute */
	protected float fLowerBound;

	/** Index position */
	protected int iPos;

	/** Set of tags for nominal attributes */
	protected Vector vecTags;

	/** Hash table for tags translation */
	protected Hashtable htTagsMap;

	//---------------------------------------------------------------------

   /** 
	 *  Builds and initializes an attribute
	 *
	 *  @param name The attribute name
	 *  @param type The attribute type
	 *  @param pos The attribute position in the data set
	 *  @param pos The tags for nominal attributes
    */
   public Attribute ( String name , int type, int pos, Vector tags )
	{
		sName   = name;
		iType   = type;
		iPos    = pos;
		vecTags = tags;

		htTagsMap = new Hashtable();
		for ( int i=0, iMax=tags.size() ; i<iMax ; i++ )
			htTagsMap.put(tags.elementAt(i), new Integer(i));
	}

   //---------------------------------------------------------------------

   /** 
	 *  Returns the name of the attribute
	 *
    *  @return the name
    */
   public String name()
	{
		return sName;
	}

   //---------------------------------------------------------------------

   /** 
	 *  Returns the type of the attribute
	 *
    *  @return the type
    */
   public int type()
	{
		return iType;
	}

	//---------------------------------------------------------------------
	 
   /**
    *  Returns the maximum value for an attribute.
    *
    *  @return the maximum value
    */
   public float maximum()
   {
      return fMaxValue;
   }  

   //---------------------------------------------------------------------

   /** 
	 *  Returns the upperBound for an attribute.
	 *
    *  @return the upper bound
    */
   public float upperBound()
	{
		return fUpperBound;
	}

	//---------------------------------------------------------------------
	 
   /**
    *  Returns the minimum value for an attribute.
    *
    *  @return the minimum value
    */
   public float minimum()
   {
      return fMinValue;
	}

   //---------------------------------------------------------------------

   /** 
	 *  Returns the lowerBound for an attribute.
	 *
    *  @return the lower bound
    */
   public float lowerBound()
	{
		return fLowerBound;
	}

	//---------------------------------------------------------------------

   /** 
	 *  Sets the maximum value for an attribute.
	 *
    *  @param max the upper bound
    */
   public void setMaximum ( float max )
	{
		fMaxValue = max;
	}
	
	//---------------------------------------------------------------------

   /** 
	 *  Sets the upperBound for an attribute.
	 *
    *  @param ub the upper bound
    */
   public void setUpperBound ( float ub )
	{
		fUpperBound = ub;
	}

	//---------------------------------------------------------------------

   /** 
	 *  Sets the minimum value for an attribute.
	 *
    *  @param min the upper bound
    */
   public void setMinimum ( float min )
	{
		fMinValue = min;
	}
	
	//---------------------------------------------------------------------

   /** 
	 *  Sets the lowerBound for an attribute.
	 *
    *  @param lb the lower bound
    */
   public void setLowerBound ( float lb )
	{
		fLowerBound = lb;
	}

   //---------------------------------------------------------------------

   /** 
	 *  Returns the attribute index position
	 *
    *  @return the index position
    */
   public int index()
	{
		return iPos;
	}

   //---------------------------------------------------------------------

   /** 
	 *  Returns the tag for the requested nominal index.
	 *
	 *  @param i the nominal index position
    *  @return the tag
    */
   public String tag ( int i )
	{
		return (String)vecTags.elementAt(i);
	}

	//---------------------------------------------------------------------

   /** 
	 *  Returns the index of nominal tag
	 *
    *  @param tag the tag
	 *  @return the nominal index position
    */
   public int tagValue ( String tag )
	{
		int  iRes = -1;
		Integer i = (Integer)htTagsMap.get(tag);

		if ( i!=null )
			iRes = i.intValue();
			
		return iRes;
	}

	//---------------------------------------------------------------------

   /** 
	 *  Returns the distance of bounds.
	 *
	 *  @return the distance
    */
   public float distance ()
	{
		float fRes = -1;

		switch ( iType ) {
			case NOMINAL: fRes = vecTags.size();          break;
			case REAL:    
			case INTEGER: fRes = fUpperBound-fLowerBound; break;
		}	

		return fRes;
	}

	//---------------------------------------------------------------------

   /** 
	 *  Returns the string representation of the attribute
	 *
    *  @return the string representation
    */
   public String toString ()
	{
		String sRes = "@ATTRIBUTE\t"+sName+"\t";

		switch ( iType ) {
			case NOMINAL: sRes += "{";
			              for ( int i=0, iMax=vecTags.size()-1 ; i<iMax ; i++ )
			                 sRes += vecTags.elementAt(i)+",";
			              sRes += vecTags.elementAt(vecTags.size()-1)+"}";
			              break;
			case REAL:    sRes += "REAL"; break;
			case INTEGER: sRes += "INTEGER ["+(int)fMinValue+","+(int)fMaxValue+"]"; break;
			default:      sRes += "UNKNOWN"; break;
		}	
		
		return sRes;
	}

   //---------------------------------------------------------------------
}
