/*------------------------------------------------------------------------------
    GALE. Fine-grained parallel Genetic Algorithms for Data Mining.
    Copyright (C) 2002  Xavier Llor <xllora@illigal.ge.uiuc.edu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
------------------------------------------------------------------------------*/


package edu.uiuc.ge.illigal.gale.data;

import java.io.*;
import java.util.*;

import edu.uiuc.ge.illigal.gale.config.*;

/**
 * This class contains the Sample Memory obtained from a file.
 * The information must be separated with some delimiter, and each
 * sample must appear on a different row.
 *
 * @author  Xavier Llor&agrave; &lt;<a href="mailto:xllora@illigal.ge.uiuc.edu">xllora@illigal.ge.uiuc.edu</a>&gt;
 * @version 0.9alpha
 * @since 0.9alpha
 */

public class FileInstanceSet 
extends InstanceSet
{
   //---------------------------------------------------------------------

	/** GALE Environment configuration object */
	protected GALEConfiguration galeCnf;

	/** Date of successful read */
	protected Date dateSTS;

	/** Date of successful read */
	protected Date dateFTS;

   //---------------------------------------------------------------------

   /** Initialize a InstaceSet from the file specified by the String
    *  information that it receives.
	 *
    *  The sample values are  described by arff format.
    *  @param galeCnf Contains the GALE configuration
    *  @param sFileName Contains the file name for the instance set
    *  @exception Exception The file could not be loaded
    */
   public FileInstanceSet (GALEConfiguration galeCnf, String sFileName)
   throws Exception
   {
		this.galeCnf = galeCnf;

		String            sCad = null;
      FileReader        frIn = new FileReader(sFileName);
      LineNumberReader lnrIn = new LineNumberReader (frIn);
      Vector          vecIns = new Vector();

		// Timestamping
		dateSTS = new Date();

		// Initializing attribute set
		attSet = new AttributeSet();

		// Reading the header
      while ( null!=(sCad=lnrIn.readLine()) ) {
			sCad = sCad.trim();
			if ( sCad.startsWith("%") )           continue;
			if ( sCad.equalsIgnoreCase("@DATA") ) break; 
			else
			 	processHeaderLine(sCad);
		}

		// Check number of attributes
		if ( attSet.size()!=galeCnf.getNumAttributes()+1 )
			throw new Exception("Wrong number of attributes");

		// Reading the instances
      while ( null!=(sCad=lnrIn.readLine()) ) {
			sCad = sCad.trim();
			if ( sCad.startsWith("%") || sCad.equals("") ) continue;
        	vecIns.addElement(sCad);
		}

      iSize = vecIns.size();
      fsInstances = new Instance[iSize];
      for ( int i=0 ; i<iSize ; i++ ) 
			fsInstances[i]=new StringInstance((String)vecIns.elementAt(i),attSet);	
      computeBounds();

		dateFTS = new Date();
   }

   //---------------------------------------------------------------------

   /** Processes a line of the header of the arff file
	 *
    *  @param sLine the line to be processed
    *  @exception Exception corrupted instance found
    */
   protected void processHeaderLine ( String sLine )
   throws Exception
   {
		StringTokenizer st = new StringTokenizer(sLine);
		String      sToken = null;
		int           iMin = 0;
		int           iMax = 0;

		while ( st.hasMoreTokens() ) {
			sToken = st.nextToken();

			if ( sToken.equalsIgnoreCase("@RELATION") )
				sRelation = st.nextToken();
			else if ( sToken.equalsIgnoreCase("@ATTRIBUTE") ) {
				String  sAtt = st.nextToken();
				int    iType = -1;
				String  sCad = st.nextToken();
				Vector   vec = new Vector();
				if ( sCad.equalsIgnoreCase("REAL")||
 				     sCad.equalsIgnoreCase("NUMERIC") ) {
					iType = Attribute.REAL;
				} 
				else if ( sCad.equalsIgnoreCase("INTEGER") ) {
					iType = Attribute.INTEGER;
					sCad  = st.nextToken();
					StringTokenizer sv = new StringTokenizer(sCad,"[,]");
					iMin = Integer.parseInt(sv.nextToken());
					iMax = Integer.parseInt(sv.nextToken());
				} 
				else if ( sCad.startsWith("{") ) {
					iType = Attribute.NOMINAL;
					StringTokenizer tag = new StringTokenizer(sCad,"{,}");
					while ( tag.hasMoreTokens() )
						vec.addElement(tag.nextToken());
				}
				else throw new Exception(sCad+" unknown attribute type");

				Attribute att = new Attribute(sAtt,iType,attSet.size(),vec);
				att.setMinimum(iMin);
				att.setMaximum(iMax);
				attSet.add(att);
			}
		}
	}

   //---------------------------------------------------------------------

   /** Checks the Sample Bounds.
	 *
    *  @exception Exception corrupted instance found
    */
   protected void computeBounds ()
   throws Exception
   {
      Instance  ins = null;
      int       iNA = attSet.size()-1;
      Attribute att = null;
      
		for ( int i=0 ; i<iNA ; i++  ) {
			att = attSet.attribute(i);
			if ( att.type()==Attribute.NOMINAL ) {
				att.setLowerBound(0);
				att.setUpperBound(att.distance());
				att.setMinimum(0);
				att.setMaximum(att.distance());
			}
			else if ( att.type()==Attribute.INTEGER ) {
				float fInc = (att.maximum()-att.minimum())/25;
				att.setLowerBound(att.minimum()-fInc);
				att.setUpperBound(att.maximum()+fInc);
			}
			else if ( att.type()==Attribute.REAL  ) {
				float fMax = fsInstances[0].getValue(i);
				float fMin = fsInstances[0].getValue(i);
				float fTmp = 0;

				for ( int j=0 ; j<fsInstances.length ; j++ ) {
					if ( fsInstances[j].isKnown(i) ) {
            		fTmp = fsInstances[j].getValue(i);            
            		if ( fTmp>fMax )      fMax = fTmp;
						else if ( fTmp<fMin ) fMin = fTmp;
					}
				}
				
				att.setMaximum(fMax);
				att.setMinimum(fMin);
				fTmp = (fMax-fMin)/25;
				att.setUpperBound(fMax+fTmp);
				att.setLowerBound(fMin-fTmp);
			}
			else
				throw new Exception ( "Unknown attribute type" );
		}
		// Class attribute
		att = attSet.attribute(iNA);
		if ( att.type()==Attribute.NOMINAL ) {
			att.setMinimum(0);
			att.setMaximum(att.distance());
			att.setLowerBound(0);
			att.setUpperBound(att.distance());
		}
		else 
			throw new Exception("Class attribute ("+att.name()+
			                    ") must be nominal");
   }
   
   //---------------------------------------------------------------------

   /** Converts the sample into a string readable format. The format is
       equivalent to the constructor format.

       @return The Sample Memory information in a String format
    */
   public String toString ()
   {
   	String sRes="";

		sRes += "%\n";
		sRes += "% GALE Environment instance set load\n";
		sRes += "% Starting  reading relation at : "+dateSTS+"\n";
		sRes += "% Finishing reading relation at : "+dateFTS+"\n";
		sRes += "%\n";
		sRes += "% Internal attribute bounds\n";
		sRes += "% -------------------------\n";

		for ( int i=0, iMax=attSet.size() ; i<iMax ; i++ ) {
			Attribute a = attSet.attribute(i);
			sRes += "% "+a.name()+"\t: "+a.minimum()+","+a.maximum()+" (";
			sRes += +a.lowerBound()+","+a.upperBound()+")\n";
		}
		sRes += "%\n";
		sRes += "% Maximal distance : "+galeCnf.getDistance().maximalDistance();
		sRes += "\n%\n";

		sRes += "%\n\n";

		sRes += "@RELATION "+sRelation+"\n\n";
		sRes += attSet+"\n";

		sRes += "@DATA\n";


     	for ( int i=0 ; i<iSize ; i++ ) {
        sRes += fsInstances[i].toString()+"\n";
		}

     	return sRes;
   }

   //---------------------------------------------------------------------

	public static void main ( String [] sArgs )
	{
		try {
			GALEConfiguration g = new FileGALEConfiguration(sArgs[0]);
			InstanceSet       i = new FileInstanceSet(g,sArgs[1]);

			g.getDistance().init(i.attributeSet());


			System.out.println(i);
		}
		catch ( Exception e ) {
			e.printStackTrace();
		}
	}

   //---------------------------------------------------------------------

}
